#include "testApp.h"

#define FRM 1

//--------------------------------------------------------------
void testApp::setup(){
    ofBackground(255,255,255);
	ofSetFrameRate(30);
	//ofSetVerticalSync(true);
	//ofSetLogLevel(OF_LOG_VERBOSE);

    width = 640;
	height = 480;
	ofw = ofGetWidth();
	ofh = ofGetHeight();

	myfont.loadFont("arial.ttf", 32);

    meshRing.setMode(OF_PRIMITIVE_TRIANGLE_STRIP);
    for (int d = 0; d <= 360; d++) {
        float r = ofDegToRad((float)d);
        float x1 = 25.0 * cos(r);
        float y1 = 25.0 * sin(r);
        float x2 = 45.0 * cos(r);
        float y2 = 45.0 * sin(r);
        meshRing.addVertex(ofPoint(x1, y1));
        meshRing.addTexCoord(ofVec2f(d / 360.0, 0));
        meshRing.addVertex(ofPoint(x2, y2));
        meshRing.addTexCoord(ofVec2f(d / 360.0, 1));
    }

    bool connected = tcpClient.setup("192.168.11.8", 16000);
    //bool connected = tcpClient.setup("127.0.0.1", 16000);
	//tcpClient.sendRaw("z\n"); // PanTilt Reset

  	// Uncomment this to show movies with alpha channels
	//roverMovie.setPixelFormat(OF_PIXELS_RGBA);
	//roverMovie.loadMovie("http://127.0.0.1:8080/video");
	roverMovie.loadMovie("http://192.168.11.8:8080/?action=stream");
	roverMovie.play();

    colorImage.allocate(width, height);
	grayImage.allocate(width, height);
	grayThres.allocate(width, height);
	grayImageFace.allocate(width/2.0, height/2.0);

    // This uses the default camera calibration and marker file
	artk.setup(width, height);

	threshold = 85;
	artk.setThreshold(threshold);

	stopPlay = false;

	posLR = 0;
	posUD = 0;
	scanL = true;
	scanR = false;
	scanLR = true;
	foundM0 = false;
	cameraM0 = false;
	roverM0 = false;
	roverMID0 = false;
	roverMID1 = false;
	roverMID2 = false;
	roverMID3 = false;
	roverMID4 = false;
	roverMID5 = false;
	frameLoss = 0;

    // GUI
    moveAround.addListener(this,&testApp::moveAroundToggled);
    faceDetect.addListener(this,&testApp::faceDetectToggled);
    findMarker.addListener(this,&testApp::findMarkerToggled);
    thres.addListener(this,&testApp::thresChanged);
    pan.addListener(this,&testApp::panChanged);
    tilt.addListener(this,&testApp::tiltChanged);

    gui.setup("ARTURo", "ARTURo.xml", ofw - 210, 10); // most of the time you don't need a name
	gui.add(moveAround.setup("moveAround", true));
	gui.add(faceDetect.setup("faceDetect", false));
	gui.add(findMarker.setup("findMarker", false));
	gui.add(thres.setup("threshold", 85, 10, 99));
	gui.add(drawThres.setup("drawThreshold", false));
	gui.add(pan.setup("pan", 0, -85, 85));
	gui.add(tilt.setup("tilt", 0, -20, 85));

	bHide = false;

    // tcpClient was here

    //finder.setup("haarcascade_frontalface_default.xml");
	//finder.setup("haarcascade_frontalface_alt.xml");
	finder.setup("haarcascade_frontalface_alt2.xml");
	//finder.setup("haarcascade_frontalface_alt_tree.xml");
	finder.setScaleHaar(1.4);
	finder.setNeighbors(2);

	img0.loadImage("mercurymap_s.jpg");
	img1.loadImage("venusmap_s.jpg");
	img2.loadImage("earthmap1k_s.jpg");
	img3.loadImage("mars_1k_color_s.jpg");
	img4.loadImage("jupitermap_s.jpg");
	img5.loadImage("saturnmap_s.jpg");
	imgRing.loadImage("texture_saturn.jpg");

    ofDisableArbTex();
    ofEnableSmoothing();
    ofEnableLighting();
    ofEnableSeparateSpecularLight();
    light.enable();

    glShadeModel(GL_SMOOTH);
}

void testApp::moveAroundToggled(bool &moveA) {
    if (moveA) {
        faceDetect = false;
        findMarker = false;

        tcpClient.sendRaw("z\n");

        posLR = 0; pan = 0;
        posUD = 0; tilt = 0;
    }
}

void testApp::faceDetectToggled(bool &faceD) {
    if (faceD) {
        moveAround = false;
        findMarker = false;

        posLR = 0; pan = 0;
        posUD = 0; tilt = 0;

        tcpClient.sendRaw("z\n");
        for (int i = 0; i < 45; i++) {
            posUD++; tilt = tilt + 1;
            tcpClient.sendRaw("u\n");
        }
    }
}

void testApp::findMarkerToggled(bool &findM) {
    if (findM) {
        moveAround = false;
        faceDetect = false;

        tcpClient.sendRaw("z\n");

        posLR = 0; pan = 0;
        posUD = 0; tilt = 0;
        scanLR = true;
        foundM0 = false;
        cameraM0 = false;
        roverM0 = false;
        roverMID0 = false;
        roverMID1 = false;
        roverMID2 = false;
        roverMID3 = false;
        roverMID4 = false;
        roverMID5 = false;
        frameLoss = 0;
    }
}

void testApp::thresChanged(int &thr) {
    threshold = thr;
    artk.setThreshold(threshold);
}

void testApp::panChanged(int &pan) {
    int pp = pan - posLR;

    if (pp > 0) {
        for (int i = 0; i < pp; i++) {
            posLR++;
            tcpClient.sendRaw("r\n");
        }
    } else if (pp < 0) {
        for (int i = pp; i < 0; i++) {
            posLR--;
            tcpClient.sendRaw("l\n");
        }
    }
}

void testApp::tiltChanged(int &tilt) {
    int pp = tilt - posUD;

    if (pp > 0) {
        for (int i = 0; i < pp; i++) {
            posUD++;
            tcpClient.sendRaw("u\n");
        }
    } else if (pp < 0) {
        for (int i = pp; i < 0; i++) {
            posUD--;
            tcpClient.sendRaw("d\n");
        }
    }
}

//--------------------------------------------------------------
void testApp::update(){

    roverMovie.update();

    if (roverMovie.isFrameNew()) {
        colorImage.setFromPixels(roverMovie.getPixels(), width, height);

        // convert our camera image to grayscale
		grayImage = colorImage;
		// apply a threshold so we can see what is going on
		grayThres = grayImage;
		grayThres.threshold(threshold);

        // Pass in the new image pixels to artk
        artk.update(grayImage.getPixels());

        if (moveAround) {

        } else if (faceDetect) {
            //grayImage.scale(0.5, 0.5);
            grayImage.resize(width/2, height/2);
            grayImageFace = grayImage;
            finder.findHaarObjects(grayImageFace, 20, 20, 280, 200, 32, 32);
            //cout << grayImage.width << " x " << grayImage.height << endl;
            grayImage.resize(width, height);
        } else if (findMarker) {
            if (!roverMID0 && !roverMID1 && !roverMID2 && !roverMID3 && !roverMID4 && !roverMID5) {
                markerID = 0;
                roverMarker(markerID);   // vai al marker 0
            }
            if (roverMID0 && !roverMID1 && !roverMID2 && !roverMID3 && !roverMID4 && !roverMID5) {
                markerID = 1;
                roverMarker(markerID);   // vai al marker 1
            }
            if (roverMID0 && roverMID1 && !roverMID2 && !roverMID3 && !roverMID4 && !roverMID5) {
                markerID = 2;
                roverMarker(markerID);   // vai al marker 2
            }
            if (roverMID0 && roverMID1 && roverMID2 && !roverMID3 && !roverMID4 && !roverMID5) {
                markerID = 3;
                roverMarker(markerID);   // vai al marker 3
            }
            if (roverMID0 && roverMID1 && roverMID2 && roverMID3 && !roverMID4 && !roverMID5) {
                markerID = 4;
                roverMarker(markerID);   // vai al marker 4
            }
            if (roverMID0 && roverMID1 && roverMID2 && roverMID3 && roverMID4 && !roverMID5) {
                markerID = 5;
                roverMarker(markerID);   // vai al marker 5
            }
        }
    }
}

//--------------------------------------------------------------
void testApp::draw(){

    int w = ofGetWidth();
	int h = ofGetHeight();

	if(w != ofw || h != ofh) {
        ofw = w;
        ofh = h;
        gui.setPosition(ofw - 210, 10);
        //ofSetWindowShape(width, height);
        cout << ofw << " " << ofh << endl;
    }

	ofSetHexColor(0xFFFFFF);

    //roverMovie.draw(0, 0);
    colorImage.draw(0, 0, ofw, ofh);
    //grayImage.draw(0, 0);

    ofSetHexColor(0x000000);

    if (findMarker) {
        int ind0 = artk.getMarkerIndex(markerID);
        int id0 = artk.getMarkerID(0);
        ofVec3f camPos = artk.getCameraPosition(ind0);
        ofVec3f camTra = artk.getTranslation(ind0);
        ofQuaternion camQua = artk.getOrientationQuaternion(ind0);
        float angle, x, y, z;
        camQua.getRotate(angle, x, y, z);

        ofVec3f center = artk.getDetectedMarkerCenter(ind0);

//        ofDrawBitmapString("pan-tilt: " + ofToString(posLR) + ", " + ofToString(posUD), 680, 280);
//        ofDrawBitmapString("campos: " + ofToString(camPos.x, 2) + ", " + ofToString(camPos.y, 2) + ", " + ofToString(camPos.z, 2), 680, 300);
//        ofDrawBitmapString("camtra: " + ofToString(camTra.x, 2) + ", " + ofToString(camTra.y, 2) + ", " + ofToString(camTra.z, 2), 680, 320);
//        ofDrawBitmapString("camrot: " + ofToString(angle, 2) + ", " + ofToString(x, 2) + ", " + ofToString(y, 2) + ", " + ofToString(z, 2), 680, 340);
//        ofDrawBitmapString("index: " + ofToString(ind0) + " id: " + ofToString(id0), 680, 360);
//        ofDrawBitmapString("center: " + ofToString(center.x, 2) + ", " + ofToString(center.y, 2) + ", " + ofToString(center.z, 2), 680, 380);
//        ofDrawBitmapString("diagonal: " + ofToString(diag), 680, 400);

    }

    ofPushView();

    if (faceDetect) {
        if (finder.blobs.size() > 0) {
            ofRectangle cur = finder.blobs[0].boundingRect;
            ofNoFill();
            ofSetLineWidth(4);
            ofSetColor(240, 30, 30, 200);
            float sfx = 2.0 * (float)ofw / (float)width;
            float sfy = 2.0 * (float)ofh / (float)height;
            ofRect(sfx * cur.x, sfy * cur.y, sfx * cur.width, sfy * cur.height);

            int fcx = 2 * (cur.x + cur.width / 2);
            int fcy = 2 * (cur.y + cur.height / 2);

            if ((fcx - width / 2) > 50) {
                //pan.removeListener(this,&testApp::panChanged);
                posLR++;
                //pan = pan + 1;
                tcpClient.sendRaw("r\n");
                //pan.addListener(this,&testApp::panChanged);
            } else if ((fcx - width / 2) < -50) {
                //pan.removeListener(this,&testApp::panChanged);
                posLR--;
                //pan = pan - 1;
                tcpClient.sendRaw("l\n");
                //pan.addListener(this,&testApp::panChanged);
            }
            if ((fcy - height / 2) > 50) {
                //tilt.removeListener(this,&testApp::tiltChanged);
                posUD--;
                //tilt = tilt - 1;
                tcpClient.sendRaw("d\n");
                //tilt.addListener(this,&testApp::tiltChanged);
            } else if ((fcy - height / 2) < -50) {
                //tilt.removeListener(this,&testApp::tiltChanged);
                posUD++;
                //tilt = tilt + 1;
                tcpClient.sendRaw("u\n");
                //tilt.addListener(this,&testApp::tiltChanged);
            }
        }
    } else if (findMarker || moveAround) {
        // ARTK draw
        // An easy was to see what is going on
        // Draws the marker location and id number
        //artk.draw(0, 0);

        int myID0 = artk.getMarkerIndex(0);
        int myID1 = artk.getMarkerIndex(1);
        int myID2 = artk.getMarkerIndex(2);
        int myID3 = artk.getMarkerIndex(3);
        int myID4 = artk.getMarkerIndex(4);
        int myID5 = artk.getMarkerIndex(5);

        // ARTK 3D stuff
        // This is another way of drawing objects aligned with the marker
        // First apply the projection matrix once
        artk.applyProjectionMatrix(ofw, ofh);
        // Find out how many markers have been detected
        int numDetected = artk.getNumDetectedMarkers();
        //ofEnableAlphaBlending();
        // Draw for each marker discovered
        for(int i=0; i<numDetected; i++) {
            // Set the matrix to the perspective of this marker
            // The origin is in the middle of the marker
            artk.applyModelMatrix(i);

            // Draw a line from the center out
            ofEnableSmoothing();
            ofSetLineWidth(2);
            ofEnableDepthTest();
            ofEnableNormalizedTexCoords();
            ofEnableAlphaBlending();

            ofPushMatrix();

            ofTranslate(0, 0, 30);
            ofRotateX(180);
            ofRotateY(ofGetFrameNum());
            ofFill();
            ofSetColor(250, 250, 250, 255);
            if (i == myID0) {
                img0.bind();
                ofDrawSphere(18);   // mercurio
                img0.unbind();
            } else if (i == myID1) {
                img1.bind();
                ofDrawSphere(22);   // venere
                img1.unbind();
            } else if (i == myID2) {
                img2.bind();
                ofDrawSphere(20);   // terra
                img2.unbind();
            } else if (i == myID3) {
                img3.bind();
                ofDrawSphere(19);   // marte
                img3.unbind();
            } else if (i == myID4) {
                img4.bind();
                ofDrawSphere(25);   // giove
                img4.unbind();
            } else if (i == myID5) {
                img5.bind();
                ofDrawSphere(23);   // saturno
                img5.unbind();

                ofSetColor(250, 250, 250, 240);
                ofPushMatrix();
                ofRotateX(90);
                imgRing.bind();
                meshRing.draw();
                imgRing.unbind();
                ofPopMatrix();
            }

            ofPopMatrix();

            ofPushMatrix();
            ofSetColor(30, 60, 250, 255);
            ofScale(0.2, -0.2, 1);
            if (i == myID0) {
                myfont.drawString("Mercurio", -100, 220);
            } else if (i == myID1) {
                myfont.drawString("Venere", -100, 220);
            } else if (i == myID2) {
                myfont.drawString("Terra", -100, 220);
            } else if (i == myID3) {
                myfont.drawString("Marte", -100, 220);
            } else if (i == myID4) {
                myfont.drawString("Giove", -100, 220);
            } else if (i == myID5) {
                myfont.drawString("Saturno", -100, 220);
            }
            ofPopMatrix();

            ofDisableAlphaBlending();
            ofDisableNormalizedTexCoords();
            ofDisableDepthTest();
            ofDisableSmoothing();
        }
    }

    ofPopView();

    ofSetColor(250, 250, 250, 255);

    if (bHide) {
		gui.draw();
	}

	if (drawThres) {
        grayThres.draw(ofw - 320, ofh - 240, 320, 240);
    }
}

//--------------------------------------------------------------
void testApp::roverMarker(int id) {

    int ind0;

    // cerca il marker con il pan
    if (scanLR && !(ofGetFrameNum() % FRM)) {

        ind0 = artk.getMarkerIndex(id);

        if (ind0 > -1) {
            scanLR = false;
            foundM0 = true;
            cout << "found marker " << id << endl;
        } else if (scanL && posLR > -85) {
            posLR--; pan = pan - 1;
            tcpClient.sendRaw("l\n");   // PanTilt sx
            //cout << "L " << posLR << " "  << newFrameCount << endl;
            if (posLR == -84) {
                scanL = false;
                scanR = true;
            }
        } else if (scanR && posLR < 85) {
            posLR++; pan = pan + 1;
            tcpClient.sendRaw("r\n");   // PanTilt dx
            //cout << "R " << posLR << " "  << newFrameCount << endl;
            if (posLR == 84) {
                scanL = true;
                scanR = false;
            }
        }
    }
    // centra il marker con il pan
    else if (foundM0 && !(ofGetFrameNum() % FRM)) {

        ind0 = artk.getMarkerIndex(id);

        if (ind0 > -1) {
            center = artk.getDetectedMarkerCenter(ind0);
            if (center.x < (width / 2) - 20 && posLR > -85) {
                posLR--; pan = pan - 1;
                tcpClient.sendRaw("l\n");   // PanTilt dx
            } else if (center.x > (width / 2) + 20 && posLR < 85) {
                posLR++; pan = pan + 1;
                tcpClient.sendRaw("r\n");   // PanTilt sx
            } else {
                foundM0 = false;
                cameraM0 = true;
                cout << "camera direction OK" << endl;
            }
        }
    }
    // allinea il corpo alla camera
    else if (cameraM0 && !(ofGetFrameNum() % FRM)) {

        ind0 = artk.getMarkerIndex(id);

        if (ind0 > -1) {
            center = artk.getDetectedMarkerCenter(ind0);
            if (center.x < (width / 2) - 60 && posLR > -85) {
                posLR--; pan = pan - 1;
                tcpClient.sendRaw("l\n");   // PanTilt sx
                //cout << "center < " << center.x << endl;
            } else if (center.x > (width / 2) + 60 && posLR < 85) {
                posLR++; pan = pan + 1;
                tcpClient.sendRaw("r\n");   // PanTilt dx
                //cout << "center > " << center.x << endl;
            } else {
                if (posLR > -2 && posLR < 2) {
                    cameraM0 = false;
                    roverM0 = true;
                    frameLoss = 0;
                    cout << "rover direction OK" << endl;
                } else if (posLR < -1) {
                    posLR++; pan = pan + 1;
                    posLR++; pan = pan + 1;
                    tcpClient.sendRaw("r\n");   // PanTilt dx
                    tcpClient.sendRaw("r\n");   // PanTilt dx
                    tcpClient.sendRaw("s\n");   // Rover sx
                } else if (posLR > 1) {
                    posLR--; pan = pan - 1;
                    posLR--; pan = pan - 1;
                    tcpClient.sendRaw("l\n");   // PanTilt sx
                    tcpClient.sendRaw("l\n");   // PanTilt sx
                    tcpClient.sendRaw("e\n");   // Rover dx
                }
            }

        } else {
            frameLoss++;
            if (frameLoss == 20) {
                scanLR = true;
                foundM0 = false;
                cameraM0 = false;
                roverM0 = false;
                frameLoss = 0;
                cout << "Frameloss Rover -> Camera" << endl;
            }
        }
    }
    // muove verso il marker fino a una distanza prefissata
    else if (roverM0 && !(ofGetFrameNum() % FRM)) {

        ind0 = artk.getMarkerIndex(id);

        if (ind0 > -1) {
            center = artk.getDetectedMarkerCenter(ind0);
            artk.getDetectedMarkerOrderedBorderCorners(ind0, corners);

            diag = (corners[0].x - corners[2].x) * (corners[0].x - corners[2].x);
            diag += (corners[0].y - corners[2].y) * (corners[0].y - corners[2].y);
            diag += (corners[1].x - corners[3].x) * (corners[1].x - corners[3].x);
            diag += (corners[1].y - corners[3].y) * (corners[1].y - corners[3].y);
            diag /= 2.0;
            diag = sqrt(diag);

            if (center.x < (width / 2) - 40 && posLR > -85) {
                //posLR--; pan = pan - 1;
                //tcpClient.sendRaw("l\n");   // PanTilt sx
                tcpClient.sendRaw("t\n");   // Rover sx
            } else if (center.x > (width / 2) + 40 && posLR < 85) {
                //posLR++; pan = pan + 1;
                //tcpClient.sendRaw("r\n");   // PanTilt dx
                tcpClient.sendRaw("f\n");   // Rover dx
            } else {
                if (posLR > -2 && posLR < 2) {
                    if (diag < 350.0) {
                        tcpClient.sendRaw("a\n");   // Rover avanti
                        tcpClient.sendRaw("a\n");   // Rover avanti
                    } else {
                        scanLR = true;
                        foundM0 = false;
                        cameraM0 = false;
                        roverM0 = false;
                        frameLoss = 0;
                        cout << "marker " << id << " OK" << endl;
                        if (id == 0) {
                            roverMID0 = true;
                        } else if (id == 1) {
                            roverMID1 = true;
                        } else if (id == 2) {
                            roverMID2 = true;
                        } else if (id == 3) {
                            roverMID3 = true;
                        } else if (id == 4) {
                            roverMID4 = true;
                        } else if (id == 5) {
                            roverMID5 = true;
                        }
                    }
                } else if (posLR < -1) {
                    posLR++; pan = pan + 1;
                    posLR++; pan = pan + 1;
                    tcpClient.sendRaw("r\n");   // PanTilt dx
                    tcpClient.sendRaw("r\n");   // PanTilt dx
                    tcpClient.sendRaw("s\n");   // Rover sx
                } else if (posLR > 1) {
                    posLR--; pan = pan - 1;
                    posLR--; pan = pan - 1;
                    tcpClient.sendRaw("l\n");   // PanTilt sx
                    tcpClient.sendRaw("l\n");   // PanTilt sx
                    tcpClient.sendRaw("e\n");   // Rover dx
                }
            }

        } else {
            frameLoss++;
            if (frameLoss == 20) {
                scanLR = true;
                foundM0 = false;
                cameraM0 = false;
                roverM0 = false;
                frameLoss = 0;
                cout << "Frameloss Rover -> Marker" << endl;
            }
        }
    }
}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){
    switch(key){
        case 'p':
            stopPlay=!stopPlay;
            if (stopPlay) {
                roverMovie.stop();
            } else {
                roverMovie.play();
            }
        break;
        case 'z':
            tcpClient.sendRaw("z\n");
            posLR = 0; pan = 0;
            posUD = 0; tilt = 0;
            scanLR = true;
            foundM0 = false;
            cameraM0 = false;
            roverM0 = false;
            roverMID0 = false;
            roverMID1 = false;
        break;
        case OF_KEY_UP:
            if (moveAround) {
                tcpClient.sendRaw("a\n");   // Rover avanti
            }
        break;
        case OF_KEY_DOWN:
            if (moveAround) {
                tcpClient.sendRaw("i\n");   // Rover indietro
            }
        break;
        case OF_KEY_LEFT:
            if (moveAround) {
                tcpClient.sendRaw("s\n");   // Rover sx
            }
        break;
        case OF_KEY_RIGHT:
            if (moveAround) {
                tcpClient.sendRaw("e\n");   // Rover dx
            }
        break;
        case 'w':
            if (moveAround && posUD < 85) {
                posUD++; tilt = tilt + 1;
                tcpClient.sendRaw("u\n");   // PanTilt up
            }
        break;
        case 's':
            if (moveAround && posUD > -20) {
                posUD--; tilt = tilt - 1;
                tcpClient.sendRaw("d\n");   // PanTilt down
            }
        break;
        case 'a':
            if (moveAround && posLR > -85) {
                posLR--; pan = pan - 1;
                tcpClient.sendRaw("l\n");   // PanTilt sx
            }
        break;
        case 'd':
            if (moveAround && posLR < 85) {
                posLR++; pan = pan + 1;
                tcpClient.sendRaw("r\n");   // PanTilt dx
            }
        break;
        case 'm':
            threshold++;
            if (threshold > 99) {
                threshold = 99;
            }
            artk.setThreshold(threshold);
        break;
        case 'n':
            threshold--;
            if (threshold < 10) {
                threshold = 10;
            }
            artk.setThreshold(threshold);
        break;
        case 'h':
            bHide = !bHide;
            gui.setPosition(ofGetWidth() - 210, 10);
        break;
        case 'f':
            ofToggleFullscreen();
        break;
    }
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){

}


//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void testApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void testApp::dragEvent(ofDragInfo dragInfo){

}
